<?php
/**
 * Namespace Teknisa\Logic\IntNutrientes\Controllers
 *
 * PHP version 5
 *
 * @category Php
 * @package  Tecfood\Integracao\Controllers
 * @author   andreteixeira <andre.teixeira@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
namespace Teknisa\Logic\IntNutrientes\Controllers;

use Geral\Entities\Organizacao;
use Teknisa\Logic\IntNutrientes\Services\RecebeProdutos as RecebeProdutosService;
use Teknisa\Logic\SIT\Request;
use Teknisa\Logic\SIT\Response;


/**
 * Class RecebeProdutos Controller
 *
 * PHP version 5
 *
 * @category Php
 * @package  Tecfood\Integracao\Controllers
 * @author   andreteixeira <andre.teixeira@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
class RecebeProdutos
{

    /**
     * Controla o Recebimento de Produtos
     * Esta classe se refere à visão Teknisa: essa ação será executada quando o
     * cliente ENVIAR os produtos (Rota: envProduto).
     *
     * @param Request  $request  Request
     * @param Response $response Response
     */
    public function recebeProdutos (Request $request, Response $response)
    {
        //Recebe e decodifica a string JSON
        $json      = $request->getData();

        //Valida se o parâmetro foi passado corretamente.
        if (empty($json) === false) {

            //Se JSON não é um array, decodifica-o.
            if (!is_array($json)) {
                $json = json_decode($json, true);
            }

            $resp = self::validaOrganizacao($json['NRORGINT']);

            if ($resp) {
                $resp = json_encode($json);

                //Realiza a inserção no banco da Teknisa
                $erro = RecebeProdutosService::recebeProdutos($json);

                //Verifica se ocorreram erros no processo de recebimento.
                if ($erro != null) {
                    $resp = $erro;
                }
            } else {
                $resp = tk_get_message_by_name('msg_ger07600_msgErroOrganizacao');
            }
        } else {
            //$resp = array('msg_ger07600_parametroIncorreto', array());
            $resp = tk_get_message_by_name('msg_ger07600_parametroIncorreto');
        }

        //Gera Response
        $response->setData($resp);
    }

    /**
     * Verifica se a Organizacao passada por parametro é válida.
     * Retorna true se a Organização existe e está ativa.
     *
     * @param String $nrorgint Organizaçao
     *
     * @return bool.
     */
    public static function validaOrganizacao ($nrorgint)
    {
        $reg = Organizacao::getObjectByWhere(
            "NRORG = ?",
            array($nrorgint)
        );

        $retorno = false;
        if ($reg instanceof Organizacao) {
            if ($reg->getIdativo() == 'S') {
                $retorno = true;
            }
        }

        return $retorno;
    }
}
